using Micro.Implementation;
using Micro.Interfaces;
using NUnit.Framework;

namespace MicroTests.Implementation
{
    [TestFixture]
    public class SymTabTest : CompilerTestBase
    {
        [Test]
        public void TestDifferentLocalsWithSameName()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  if main = 0");
            sw.WriteLine("    int x");
            sw.WriteLine("    x = 5");
            sw.WriteLine("    call put_i <- x end");
            sw.WriteLine("  else");
            sw.WriteLine("    char x");
            sw.WriteLine("    x = 'a'");
            sw.WriteLine("    call put_c <- x end");
            sw.WriteLine("  end");
            sw.WriteLine("end");
            Parse();
        }


        [Test]
        public void TestLocalAndArgWithSameName()
        {
            sw.WriteLine("function int foo <- int i");
            sw.WriteLine("  int i");
            sw.WriteLine("  foo = i");
            sw.WriteLine("end");
            sw.WriteLine("function int main");
            sw.WriteLine("  int i i = 0");
            sw.WriteLine("  call foo <- i end");
            sw.WriteLine("end");
            Parse();
        }

        [Test]
        public void TestVariableDeclaredTwice1()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  int i int i");
            sw.WriteLine("end");
            try {
                Parse();
                Assert.Fail(
                    "Variable i has been declared twice in the same scope");
            }
            catch (PanicException exc) {
                Assert.AreEqual("Error: i declared twice", exc.Message);
            }
        }

        [Test]
        public void TestVariableDeclaredTwice2()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  int i");
            sw.WriteLine("  if i > 5");
            sw.WriteLine("    int i i = 3");
            sw.WriteLine("  end");
            sw.WriteLine("end");
            try {
                Parse();
                Assert.Fail(
                    "Variable i is available from this scope and must not declared twice");
            }
            catch (PanicException exc) {
                Assert.AreEqual("Error: i declared twice", exc.Message);
            }
        }


        [Test]
        public void TestVariableNotFound1()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  if 5 > 3 char c c = 'a' end");
            sw.WriteLine("  call put_c <- c end ");
            sw.WriteLine("end");
            try {
                Parse();
                Assert.Fail("Variable c should be out of scope");
            }
            catch (PanicException exc) {
                Assert.AreEqual("Error: Variable not found: 'c'", exc.Message);
            }
        }


        [Test]
        public void TestVariableNotFound2()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  c = 'a'");
            sw.WriteLine("end");
            try {
                Parse();
                Assert.Fail("Variable c never gets declared");
            }
            catch (PanicException exc) {
                Assert.AreEqual("Error: Variable not found: 'c'", exc.Message);
            }
        }

        public void TestConstantAssignment()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("INT <- 23");
            sw.WriteLine("main = INT");
            sw.WriteLine("call put_i <- INT end");
            sw.WriteLine("INT = 2");
            sw.WriteLine("main = main + INT");
            sw.WriteLine("call put_i <- INT end");
            sw.WriteLine("end");
            try {
                Parse();
                Assert.Fail(
                    "Constant can not be the left hand side of an assignment.");
            }
            catch (PanicException exc) {
                Assert.AreEqual(
                    "Error: Left-hand side of an assignment must be a variable",
                    exc.Message);
            }
        }


        private void Parse()
        {
            sw.Flush();
            source.Position = 0;
            IError err = new PanicError();
            IScanner s = new Scanner(source, err);
            IParser p = new Parser(s, err, new CodeGen("dummy", err));
            p.Parse();
        }
    }
}